@@ -21,26 +21,26 @@ from typing import (
     get_origin,
 )
 
-from smartparams.cli import Print, parse_arguments
-from smartparams.io import load_data, print_data, save_data
-from smartparams.options import Option
-from smartparams.utils import (
-    check_key_is_in,
-    check_missings,
-    check_overrides,
-    check_typings,
-    convert_to_primitive_types,
-    flatten_keys,
-    get_nested_dictionary_and_key,
-    get_object_name,
-    import_callable,
-    join_keys,
-    join_objects,
-    parse_object,
-    parse_param,
-    remove_duplicated_key_and_value,
-    str_to_bool,
+import smartparams.utils.dictionary as dictutil
+import smartparams.utils.string as strutil
+import smartparams.utils.typing as typeutil
+from smartparams.exceptions import (
+    ConfigurationError,
+    DummyError,
+    MissingPath,
+    ObjectInvalidOptionError,
+    ObjectNotRegistered,
+    UnexpectedTypeOptionArguments,
 )
+from smartparams.register import SmartRegister
+from smartparams.utils.cli import (
+    Arguments,
+    Print,
+    create_argument_parser,
+    parse_arguments,
+)
+from smartparams.utils.enums import Option
+from smartparams.utils.io import load_data, print_data, save_data
 
 T = TypeVar('T')
 
@@ -91,63 +91,72 @@ class Smart(Generic[T]):
         missing_value: Value assigned to unknown types when creating a representation.
             Can be set by env variable SMARTPARAMS_MISSING_VALUE, default '???'.
         check_missings: Whether to check missing values before instantiating object.
-            Can be set by env variable SMARTPARAMS_CHECK_MISSINGS, default 'true'.
+            Can be set by env variable SMARTPARAMS_CHECK_MISSINGS, default 'yes'.
         check_typings: Whether to check arguments type before instantiating object.
-            Can be set by env variable SMARTPARAMS_CHECK_TYPINGS, default 'true'.
+            Can be set by env variable SMARTPARAMS_CHECK_TYPINGS, default 'yes'.
         check_overrides: Whether to check override arguments before instantiating object.
-            Can be set by env variable SMARTPARAMS_CHECK_OVERRIDES, default 'true'.
+            Can be set by env variable SMARTPARAMS_CHECK_OVERRIDES, default 'yes'.
         allow_only_registered_classes: Whether to allow import only registered classes.
-            Can be set by env variable SMARTPARAMS_ALLOW_ONLY_REGISTERED_CLASSES, default 'false'.
+            Can be set by env variable SMARTPARAMS_ALLOW_ONLY_REGISTERED_CLASSES, default 'no'.
         strict: Whether to raise exceptions instead of warnings.
-            Can be set by env variable SMARTPARAMS_STRICT, default 'false'.
+            Can be set by env variable SMARTPARAMS_STRICT, default 'no'.
+        debug: Whether to print exception stack trace instead of cli error message.
+            Can be set by env variable SMARTPARAMS_DEBUG, default 'no'.
 
     """
 
     keyword: str = os.getenv('SMARTPARAMS_KEYWORD', default='class')
     missing_value: str = os.getenv('SMARTPARAMS_MISSING_VALUE', default='???')
 
-    check_missings: bool = str_to_bool(os.getenv('SMARTPARAMS_CHECK_MISSINGS', default='true'))
-    check_typings: bool = str_to_bool(os.getenv('SMARTPARAMS_CHECK_TYPINGS', default='true'))
-    check_overrides: bool = str_to_bool(os.getenv('SMARTPARAMS_CHECK_OVERRIDES', default='true'))
+    check_missings: bool = strutil.to_bool(os.getenv('SMARTPARAMS_CHECK_MISSINGS', default='yes'))
+    check_typings: bool = strutil.to_bool(os.getenv('SMARTPARAMS_CHECK_TYPINGS', default='yes'))
+    check_overrides: bool = strutil.to_bool(os.getenv('SMARTPARAMS_CHECK_OVERRIDES', default='yes'))
 
-    allow_only_registered_classes: bool = str_to_bool(
-        os.getenv('SMARTPARAMS_ALLOW_ONLY_REGISTERED_CLASSES', default='false'),
+    allow_only_registered_classes: bool = strutil.to_bool(
+        os.getenv('SMARTPARAMS_ALLOW_ONLY_REGISTERED_CLASSES', default='no'),
     )
 
-    strict: bool = str_to_bool(os.getenv('SMARTPARAMS_STRICT', default='false'))
+    strict: bool = strutil.to_bool(os.getenv('SMARTPARAMS_STRICT', default='no'))
+    debug: bool = strutil.to_bool(os.getenv('SMARTPARAMS_DEBUG', default='no'))
 
-    _aliases: Dict[str, str] = dict()
-    _origins: Dict[str, str] = dict()
+    __register = SmartRegister()
 
     def __init__(
         self,
-        _class: Callable[..., T] = cast(Callable[..., T], dict),
+        _callable: Callable[..., T] = cast(Callable[..., T], dict),
         /,
         **kwargs: Any,
     ) -> None:
         """Creates instance of Smart class.
 
         Args:
-            _class: Class to be wrapped.
-            **kwargs: Partial keyword arguments to be passed to the class constructor.
+            _callable: Callable object to be wrapped.
+            **kwargs: Partial keyword arguments to be passed to the callable object.
 
         """
-        self._class: Callable[..., T] = _class
-        self._params: Dict[str, Any] = dict()
+        if not callable(_callable):
+            raise TypeError("Object is not callable.")
 
         self._location: str = ''
 
-        for k, v in kwargs.items():
-            self.set(k, v)
+        self._callable = _callable
+        self._params: Dict[str, Any] = dict()
+
+        for key, value in kwargs.items():
+            self.set(key, value)
 
     @property
-    def type(self) -> Callable[..., T]:
-        return self._class
+    def type(self) -> Type[T]:
+        return typeutil.get_return_type(self._callable)
 
     @property
     def dict(self) -> Dict[str, Any]:
         return self._params.copy()
 
+    @property
+    def register(self) -> SmartRegister:
+        return self.__register
+
     def __call__(
         self,
         *args: Any,
@@ -163,15 +172,11 @@ class Smart(Generic[T]):
             An class instance.
 
         """
-        params = self._init_dict(
-            dictionary=self._params,
-            location=self._location,
-        )
-
-        callable_name = join_objects(self._location, get_object_name(self._class))
+        callable_name = strutil.join_objects(self._location, typeutil.get_name(self._callable))
+        params = self.dict
 
         if self.check_overrides:
-            check_overrides(
+            typeutil.check_overrides(
                 params=params,
                 kwargs=kwargs,
                 callable_name=callable_name,
@@ -180,18 +185,18 @@ class Smart(Generic[T]):
 
         params.update(kwargs)
 
-        return self._instantiate(
-            location=self._location,
+        return self._init(
             callable_name=callable_name,
-            callable_=self._class,
+            callable_=self._callable,
+            location=self._location,
             args=args,
             kwargs=params,
         )
 
     def __str__(self) -> str:
-        class_string = "" if self._class is dict else f"[{get_object_name(self._class)}]"
+        callable_string = "" if self._callable is dict else f"[{typeutil.get_name(self._callable)}]"
         params_string = ", ".join((f"{k}={v}" for k, v in self._params.items()))
-        return f"{self.__class__.__name__}{class_string}({params_string})"
+        return f"{Smart.__name__}{callable_string}({params_string})"
 
     def __repr__(self) -> str:
         return str(self)
@@ -211,7 +216,7 @@ class Smart(Generic[T]):
             Keys from dictionary.
 
         """
-        keys = flatten_keys(self._params) if flatten else self._params
+        keys = dictutil.flatten_keys(self._params) if flatten else self._params
         if pattern is None:
             yield from keys
         else:
@@ -232,7 +237,7 @@ class Smart(Generic[T]):
             Values from dictionary.
 
         """
-        return (self.get(k) for k in self.keys(flatten, pattern))
+        yield from (self.get(k) for k in self.keys(flatten, pattern))
 
     def items(
         self,
@@ -249,7 +254,7 @@ class Smart(Generic[T]):
             Items from dictionary.
 
         """
-        return ((k, self.get(k)) for k in self.keys(flatten, pattern))
+        yield from ((k, self.get(k)) for k in self.keys(flatten, pattern))
 
     def isin(
         self,
@@ -264,7 +269,10 @@ class Smart(Generic[T]):
             True if name is in dictionary, otherwise False.
 
         """
-        return check_key_is_in(name, self._params)
+        return dictutil.check_key_is_in(
+            key=name,
+            dictionary=self._params,
+        )
 
     def get(
         self,
@@ -284,7 +292,7 @@ class Smart(Generic[T]):
             ValueError if name doesn't exist and default value not specified.
 
         """
-        dictionary, key = get_nested_dictionary_and_key(
+        dictionary, key = dictutil.find_nested(
             dictionary=self._params,
             name=name,
             required=default is ...,
@@ -306,7 +314,7 @@ class Smart(Generic[T]):
             The given value.
 
         """
-        dictionary, key = get_nested_dictionary_and_key(
+        dictionary, key = dictutil.find_nested(
             dictionary=self._params,
             name=name,
             set_mode=True,
@@ -332,7 +340,7 @@ class Smart(Generic[T]):
             ValueError if name doesn't exist and default value not specified.
 
         """
-        dictionary, key = get_nested_dictionary_and_key(
+        dictionary, key = dictutil.find_nested(
             dictionary=self._params,
             name=name,
             required=default is ...,
@@ -357,12 +365,11 @@ class Smart(Generic[T]):
             ValueError if name doesn't exist.
 
         """
-        dictionary, key = get_nested_dictionary_and_key(
+        dictionary, key = dictutil.find_nested(
             dictionary=self._params,
             name=name,
             required=True,
         )
-
         dictionary[key] = value = function(dictionary[key])
         return value
 
@@ -394,7 +401,7 @@ class Smart(Generic[T]):
         elif isinstance(source, Mapping):
             smart = Smart(**source)
         elif isinstance(source, Sequence):
-            smart = Smart(**dict(map(parse_param, source)))
+            smart = Smart(**dict(map(strutil.parse_param, source)))
         elif isinstance(source, Path):
             smart = Smart(**load_data(source))
         else:
@@ -453,13 +460,13 @@ class Smart(Generic[T]):
         """
         if name is None:
             obj = self._init_object(
-                obj=self.dict,
+                obj=self._params,
                 location=self._location,
             )
         else:
             obj = self._init_object(
                 obj=self.get(name),
-                location=join_keys(self._location, name),
+                location=strutil.join_keys(self._location, name),
             )
 
             if persist:
@@ -489,53 +496,17 @@ class Smart(Generic[T]):
 
         smart.update_from(
             source=self._object_representation(
-                obj=self._class,
+                obj=self._callable,
                 skip_default=skip_defaults,
             ),
             override=False,
         )
 
-        return convert_to_primitive_types(
+        return typeutil.convert_to_primitive_types(
             obj=smart.dict,
             missing_value=self.missing_value,
         )
 
-    @classmethod
-    def register(
-        cls,
-        classes: Union[
-            Sequence[Union[str, Callable]],
-            Mapping[str, str],
-            Mapping[Callable, str],
-            Mapping[Union[str, Callable], str],
-        ],
-        prefix: str = '',
-    ) -> Type['Smart']:
-        """Registers classes to be imported.
-
-        Args:
-            classes: Class list or dict with aliases to be registered.
-            prefix: Prefix string added to alias.
-
-        Returns:
-            Smart object.
-
-        """
-        if isinstance(classes, Sequence):
-            cls._register_classes(
-                classes=classes,
-                prefix=prefix,
-            )
-        elif isinstance(classes, Mapping):
-            cls._register_aliases(
-                aliases=classes,
-                prefix=prefix,
-            )
-        else:
-            raise TypeError(f"Register classes type '{type(classes)}' is not supported.")
-
-        return cls
-
     def run(
         self,
         function: Optional[Callable[['Smart'], Any]] = None,
@@ -551,21 +522,37 @@ class Smart(Generic[T]):
             Smart object.
 
         """
-        args = parse_arguments(
+        parser = create_argument_parser(
             default_path=path,
         )
+        args = parse_arguments(parser)
 
-        if args.strict:
-            Smart.strict = True
+        try:
+            self._run(
+                function=function,
+                args=args,
+            )
+        except DummyError if self.debug else ConfigurationError as e:
+            parser.error(e.message)
 
-        if args.path:
+        return self
+
+    def _run(
+        self,
+        function: Optional[Callable[['Smart'], Any]],
+        args: Arguments,
+    ) -> None:
+        Smart.strict = Smart.strict or args.strict
+        Smart.debug = Smart.debug or args.debug
+
+        if args.path and args.path.exists():
             self.update_from(args.path)
 
         self.update_from(args.params)
 
         if args.dump:
             if not args.path:
-                raise ValueError("Cannot dump params if path is not specified.")
+                raise MissingPath()
 
             save_data(
                 data=self.representation(
@@ -583,6 +570,14 @@ class Smart(Generic[T]):
                     ),
                     fmt=args.format,
                 )
+            elif args.print == Print.DICT:
+                print_data(
+                    data=typeutil.convert_to_primitive_types(
+                        obj=self._params,
+                        missing_value=self.missing_value,
+                    ),
+                    fmt=args.format,
+                )
             elif args.print == Print.KEYS:
                 print_data(
                     data=tuple(self.keys(flatten=True)),
@@ -595,8 +590,6 @@ class Smart(Generic[T]):
         else:
             function(self)
 
-        return self
-
     def _init_object(
         self,
         obj: Any,
@@ -604,7 +597,7 @@ class Smart(Generic[T]):
     ) -> Any:
         if isinstance(obj, dict):
             if self.keyword in obj:
-                return self._instantiate_from_dict(
+                return self._init_from_dict(
                     dictionary=obj,
                     location=location,
                 )
@@ -630,7 +623,7 @@ class Smart(Generic[T]):
         return {
             key: self._init_object(
                 obj=value,
-                location=join_keys(location, key),
+                location=strutil.join_keys(location, key),
             )
             for key, value in dictionary.items()
         }
@@ -643,75 +636,80 @@ class Smart(Generic[T]):
         return [
             self._init_object(
                 obj=element,
-                location=join_keys(location, str(index)),
+                location=strutil.join_keys(location, str(index)),
             )
             for index, element in enumerate(lst)
         ]
 
-    def _instantiate_from_dict(
+    def _init_from_dict(
         self,
         dictionary: Dict[str, Any],
         location: str,
     ) -> Any:
-        kwargs, name, option = parse_object(
+        kwargs, name, option = typeutil.parse_object(
             dictionary=dictionary,
             keyword=self.keyword,
         )
 
-        if name == self.__class__.__name__:
-            return self._instantiate(
-                location=location,
-                callable_name=join_objects(location, self.__class__.__name__),
+        if name == Smart.__name__:
+            return self._init(
+                callable_name=strutil.join_objects(location, Smart.__name__),
                 callable_=Smart,
+                location=location,
                 kwargs=kwargs,
             )
 
-        if name in self._origins:
-            name = self._origins[name]
+        if name in self.__register.origins:
+            name = self.__register.origins[name]
         elif self.allow_only_registered_classes:
-            raise ImportError(f"Class '{name}' is not registered.")
+            raise ObjectNotRegistered(name, location)
 
-        callable_ = import_callable(name)
-        callable_name = join_objects(location, get_object_name(callable_))
+        callable_ = typeutil.import_callable(name, location=location)
+        callable_name = strutil.join_objects(location, typeutil.get_name(callable_))
 
         if option:
             if option == Option.SMART:
-                return self._instantiate(
-                    location=location,
+                return self._init(
                     callable_name=callable_name,
                     callable_=Smart,
+                    location=location,
                     args=(callable_,),
                     kwargs=kwargs,
                 )
 
             if option == Option.TYPE:
                 if kwargs:
-                    raise ValueError(f"Cannot specify any arguments for {callable_name}'s type.")
+                    raise UnexpectedTypeOptionArguments(callable_name)
 
                 return callable_
 
-            raise ValueError(f"Option '{option}' is not supported.")
+            raise ObjectInvalidOptionError(option, location)
 
-        return self._instantiate(
-            location=location,
+        return self._init(
             callable_name=callable_name,
             callable_=callable_,
+            location=location,
             kwargs=kwargs,
         )
 
-    def _instantiate(
+    def _init(
         self,
-        location: str,
         callable_name: str,
         callable_: Callable,
+        location: str,
         args: Optional[Tuple[Any, ...]] = None,
         kwargs: Optional[Dict[str, Any]] = None,
     ) -> Any:
         args = args or tuple()
         kwargs = kwargs or dict()
 
+        kwargs = self._init_dict(
+            dictionary=kwargs,
+            location=location,
+        )
+
         if self.check_missings:
-            check_missings(
+            typeutil.check_missings(
                 kwargs=kwargs,
                 missing_value=self.missing_value,
                 callable_name=callable_name,
@@ -719,7 +717,7 @@ class Smart(Generic[T]):
             )
 
         if self.check_typings:
-            check_typings(
+            typeutil.check_typings(
                 callable_=callable_,
                 args=args,
                 kwargs=kwargs,
@@ -728,14 +726,14 @@ class Smart(Generic[T]):
             )
 
         try:
-            callable_ = callable_(*args, **kwargs)
+            obj = callable_(*args, **kwargs)
         except Exception as e:
-            raise RuntimeError(f"Error during instantiate '{callable_name}'.") from e
+            raise RuntimeError(f"Error during instantiate '{callable_name}'; {e}") from e
         else:
-            if isinstance(callable_, Smart):
-                callable_._location = location
+            if isinstance(obj, Smart):
+                obj._location = location
 
-            return callable_
+            return obj
 
     def _object_representation(
         self,
@@ -757,7 +755,7 @@ class Smart(Generic[T]):
             ):
                 if annotation is Smart or isinstance(default, Smart) and default.type is dict:
                     representation[name] = {
-                        self.keyword: self.__class__.__name__,
+                        self.keyword: Smart.__name__,
                     }
                 elif get_origin(annotation) is Smart or isinstance(default, Smart):
                     if isinstance(default, Smart):
@@ -766,8 +764,8 @@ class Smart(Generic[T]):
                         param_type, *_ = get_args(annotation)
 
                     keyword = inspect.formatannotation(param_type)
-                    keyword = self._aliases.get(keyword, keyword)
-                    keyword = join_objects(keyword, Option.SMART.value)
+                    keyword = self.__register.aliases.get(keyword, keyword)
+                    keyword = strutil.join_objects(keyword, Option.SMART.value)
 
                     representation[name] = {
                         self.keyword: keyword,
@@ -785,7 +783,7 @@ class Smart(Generic[T]):
                         representation[name] = annotation.__name__ + self.missing_value
                     else:
                         keyword = inspect.formatannotation(annotation)
-                        keyword = self._aliases.get(keyword, keyword)
+                        keyword = self.__register.aliases.get(keyword, keyword)
                         representation[name] = {
                             self.keyword: keyword,
                             **self._object_representation(
@@ -797,47 +795,3 @@ class Smart(Generic[T]):
                     representation[name] = self.missing_value
 
         return representation
-
-    @classmethod
-    def _register_classes(
-        cls,
-        classes: Sequence[Union[str, Callable]],
-        prefix: str = '',
-    ) -> None:
-        cls._register_aliases(
-            aliases={c: c if isinstance(c, str) else get_object_name(c) for c in classes},
-            prefix=prefix,
-        )
-
-    @classmethod
-    def _register_aliases(
-        cls,
-        aliases: Union[
-            Mapping[str, str],
-            Mapping[Callable, str],
-            Mapping[Union[str, Callable], str],
-        ],
-        prefix: str = '',
-    ) -> None:
-        for origin, alias in aliases.items():
-            origin = origin if isinstance(origin, str) else inspect.formatannotation(origin)
-            alias = join_keys(prefix, alias)
-
-            remove_duplicated_key_and_value(
-                key=origin,
-                key_dict=cls._aliases,
-                value_dict=cls._origins,
-                message=f"Origin '{origin}' has been overridden.",
-                raise_error=cls.strict,
-            )
-
-            remove_duplicated_key_and_value(
-                key=alias,
-                key_dict=cls._origins,
-                value_dict=cls._aliases,
-                message=f"Alias '{alias}' has been overridden.",
-                raise_error=cls.strict,
-            )
-
-            cls._aliases[origin] = alias
-            cls._origins[alias] = origin
