@@ -4,6 +4,13 @@ from pathlib import Path
 from unittest.mock import Mock, patch
 
 from smartparams import Smart
+from smartparams.exceptions import (
+    ArgumentTypeError,
+    MissingArgument,
+    MissingArgumentValue,
+    MissingPath,
+    UnexpectedArgument,
+)
 from tests.custom_classes import (
     Class,
     ClassChild,
@@ -33,7 +40,7 @@ class TestSerialization(IntegrationCase):
             },
         )
 
-    def test_pickle(self) -> None:
+    def test__pickle(self) -> None:
         pickled = pkl.dumps(self.smart)
         smart = pkl.loads(pickled)
 
@@ -46,38 +53,46 @@ class TestSerialization(IntegrationCase):
 class TestSmartRunCase(IntegrationCase):
     def setUp(self) -> None:
         self.smart = Smart(Class, arg2=10)
+        self.smart.debug = True
 
     def tearDown(self) -> None:
         Smart.strict = False
 
-    @patch('smartparams.cli.sys', Mock(argv='script.py --dump --merge-params'.split()))
+    @patch('smartparams.utils.cli.sys', Mock(argv='script.py --dump --merge-params'.split()))
     def test_run__dump(self) -> None:
         with tempfile.NamedTemporaryFile(mode='w+', suffix='.yaml') as file:
             self.smart.run(function=Mock(), path=Path(file.name))
 
             self.assertEqual("arg2: 10\narg1: str???\n", file.read())
 
-    @patch('smartparams.cli.sys', Mock(argv='script.py --dump'.split()))
+    @patch('smartparams.utils.cli.sys', Mock(argv='script.py --dump'.split()))
     def test_run__dump_no_path(self) -> None:
-        self.assertRaises(ValueError, self.smart.run, function=Mock(), path=None)
+        self.assertRaises(MissingPath, self.smart.run, function=Mock(), path=None)
 
-    @patch('smartparams.cli.sys', Mock(argv='script.py --print params'.split()))
-    @patch('smartparams.io.print')
+    @patch('smartparams.utils.cli.sys', Mock(argv='script.py --print params'.split()))
+    @patch('smartparams.utils.io.print')
     def test_run__print_params(self, print_mock: Mock) -> None:
         self.smart.run(function=Mock())
 
         print_mock.assert_called_with("arg1: str???\narg2: 5\n")
 
-    @patch('smartparams.cli.sys', Mock(argv='script.py --print keys'.split()))
-    @patch('smartparams.io.print')
+    @patch('smartparams.utils.cli.sys', Mock(argv='script.py --print dict'.split()))
+    @patch('smartparams.utils.io.print')
+    def test_run__print_dict(self, print_mock: Mock) -> None:
+        self.smart.run(function=Mock())
+
+        print_mock.assert_called_with("arg2: 10\n")
+
+    @patch('smartparams.utils.cli.sys', Mock(argv='script.py --print keys'.split()))
+    @patch('smartparams.utils.io.print')
     def test_run__print_keys(self, print_mock: Mock) -> None:
         self.smart.run(function=Mock())
 
         print_mock.assert_called_with("- arg2\n")
 
-    @patch('smartparams.cli.sys', Mock(argv='script.py --strict arg1=10'.split()))
+    @patch('smartparams.utils.cli.sys', Mock(argv='script.py --strict arg1=10'.split()))
     def test_run__strict(self) -> None:
-        self.assertRaises(TypeError, self.smart.run, function=lambda x: x())
+        self.assertRaises(ArgumentTypeError, self.smart.run, function=lambda x: x())
 
     @patch('smartparams.smart.load_data', Mock(return_value={'arg1': 'string'}))
     def test_run__function(self) -> None:
@@ -92,17 +107,26 @@ class TestSmartRunCase(IntegrationCase):
 
     def test_run__without_function(self) -> None:
         function = Mock()
-        self.smart._class = function
+        self.smart._callable = function
 
         self.smart.run()
 
         function.assert_called_once_with(arg2=10)
 
+    def test_run__parser_error(self) -> None:
+        function = Mock(side_effect=MissingArgument('name', 'message'))
+        self.smart.debug = False
+
+        self.assertRaises(SystemExit, self.smart.run, function=function)
+
 
 class TestSmartRepresentationCase(IntegrationCase):
     def setUp(self) -> None:
         self.smart: Smart = Smart()
 
+    def tearDown(self) -> None:
+        self.smart.register.reset()
+
     def test_object_representation__with_defaults(self) -> None:
         class_name = f"{Class.__module__}.{Class.__qualname__}"
         class_child_name = f"{ClassChild.__module__}.{ClassChild.__qualname__}"
@@ -150,8 +174,7 @@ class TestSmartRepresentationCase(IntegrationCase):
         class_name = f"{Class.__module__}.{Class.__qualname__}"
         class_child_name = f"{ClassChild.__module__}.{ClassChild.__qualname__}"
         self.smart.allow_only_registered_classes = True
-        self.smart._aliases = {class_name: 'Parent', class_child_name: 'Child'}
-        self.smart._origins = {v: k for k, v in self.smart._aliases.items()}
+        self.smart.register({class_name: 'Parent', class_child_name: 'Child'})
         expected = {
             'vanilla_cls': {'class': 'Parent', 'arg1': 'str???', 'arg2': 5},
             'smart_cls': {'class': 'Parent:Smart', 'arg1': 'str???', 'arg2': 5},
@@ -192,9 +215,9 @@ class TestCheckCase(IntegrationCase):
         self.smart.check_overrides = False
         self.smart.check_typings = False
 
-        self.assertRaises(ValueError, self.smart, arg2='88')
+        self.assertRaises(MissingArgumentValue, self.smart, arg2='88')
 
-    @patch('smartparams.utils.warnings')
+    @patch('smartparams.utils.typing.warnings')
     def test_init_class__check_missings_warning(self, warnings: Mock) -> None:
         self.smart.strict = False
 
@@ -212,9 +235,9 @@ class TestCheckCase(IntegrationCase):
         self.smart.check_overrides = True
         self.smart.check_typings = False
 
-        self.assertRaises(ValueError, self.smart, arg2='88')
+        self.assertRaises(UnexpectedArgument, self.smart, arg2='88')
 
-    @patch('smartparams.utils.warnings')
+    @patch('smartparams.utils.typing.warnings')
     def test_init_class__check_overrides_warning(self, warnings: Mock) -> None:
         self.smart.strict = False
 
@@ -232,9 +255,9 @@ class TestCheckCase(IntegrationCase):
         self.smart.check_overrides = False
         self.smart.check_typings = True
 
-        self.assertRaises(TypeError, self.smart, arg2='88')
+        self.assertRaises(ArgumentTypeError, self.smart, arg2='88')
 
-    @patch('smartparams.utils.warnings')
+    @patch('smartparams.utils.typing.warnings')
     def test_init_class__check_typings_warning(self, warnings: Mock) -> None:
         self.smart.strict = False
 
